"""Reusable widgets for the TUI."""

from __future__ import annotations

from ytm_player.ui.widgets.album_art import AlbumArt
from ytm_player.ui.widgets.progress_bar import PlaybackProgress
from ytm_player.ui.widgets.track_table import TrackTable

__all__ = ["AlbumArt", "PlaybackProgress", "TrackTable"]
